<?php 
$OC_Version = array(26,0,11,2);
$OC_VersionString = '26.0.11';
$OC_Edition = '';
$OC_Channel = 'stable';
$OC_VersionCanBeUpgradedFrom = array (
  'nextcloud' => 
  array (
    '25.0' => true,
    '26.0' => true,
  ),
  'owncloud' => 
  array (
    '10.13' => true,
  ),
);
$OC_Build = '2024-01-25T10:35:47+00:00 9170e2d392ac59865c252eafea4c2c83c3790ceb';
$vendor = 'nextcloud';
